#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-or-later
#
# Copyright (C) 2025, Bryan O'Donoghue
# Copyright (C) 2026, Ideas on Board Oy
#
# Generate a C header file containing hex-encoded shader sources

import argparse
import binascii
import math
import pathlib
import sys


def process_file(path, out):
    data = open(path, 'rb').read()

    hex_data = [f'0x{c:02x}' for c in data]
    var_name = path.name.replace('.', '_')

    out.write(f'static constexpr std::array<unsigned char, {len(data)}> {var_name}{{\n')

    for i in range(math.ceil(len(data) / 16)):
        out.write('\t')
        out.write(', '.join(hex_data[16 * i:16 * (i + 1)]))
        out.write(',\n')

    out.write('};\n')


def main(argv):
    parser = argparse.ArgumentParser(
        description='Generate a C header file containing hex-encoded shader sources')
    parser.add_argument('--output', '-o', metavar='file', default=sys.stdout,
                        type=argparse.FileType('w', encoding='utf-8'),
                        help='Output file name. Defaults to standard output if not specified.')
    parser.add_argument('inputs', nargs='+', type=pathlib.Path,
                        help='Input file names.')
    args = parser.parse_args(argv[1:])

    args.output.write('''\
/* SPDX-License-Identifier: LGPL-2.1-or-later */
/* This file is auto-generated, do not edit! */

#pragma once

#include <array>

/*
 * List the names of the shaders at the top of header for readability's sake.
 *
''')

    for path in args.inputs:
        name = path.name.replace('.', '_')
        args.output.write(f' * - {name}\n')

    args.output.write('''\
 */

/* Hex encoded shader data */
''')

    for path in args.inputs:
        process_file(path, args.output)
        args.output.write('\n')

    return 0


if __name__ == '__main__':
    sys.exit(main(sys.argv))




#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-or-later
#
# Generate Python format definitions from YAML

import argparse
import string
import sys
import yaml


def generate(formats):
    fmts = []

    for format in formats:
        name, format = format.popitem()
        fmts.append(f'\t\t.def_readonly_static("{name}", &libcamera::formats::{name})')

    return {'formats': '\n'.join(fmts)}


def main(argv):
    parser = argparse.ArgumentParser()
    parser.add_argument('-o', dest='output', metavar='file', default=sys.stdout,
                        type=argparse.FileType('w', encoding='utf-8'),
                        help='Output file name. Defaults to standard output if not specified.')
    parser.add_argument('input', type=argparse.FileType('rb'),
                        help='Input file name.')
    parser.add_argument('template', type=argparse.FileType('r', encoding='utf-8'),
                        help='Template file name.')
    args = parser.parse_args(argv[1:])

    formats = yaml.safe_load(args.input)['formats']

    data = generate(formats)
    data = string.Template(args.template.read()).substitute(data)

    args.output.write(data)

    return 0


if __name__ == '__main__':
    sys.exit(main(sys.argv))



# SPDX-License-Identifier: BSD-2-Clause
#
# Copyright (C) 2023, Raspberry Pi Ltd
#
# Program to convert from RGB to LAB color space

def RGB_to_LAB(RGB):  # where RGB is a 1x3 array.   e.g RGB = [100, 255, 230]
    num = 0
    XYZ = [0, 0, 0]
    # converted all the three R, G, B to X, Y, Z
    X = RGB[0] * 0.4124 + RGB[1] * 0.3576 + RGB[2] * 0.1805
    Y = RGB[0] * 0.2126 + RGB[1] * 0.7152 + RGB[2] * 0.0722
    Z = RGB[0] * 0.0193 + RGB[1] * 0.1192 + RGB[2] * 0.9505

    XYZ[0] = X / 255 * 100
    XYZ[1] = Y / 255 * 100  # XYZ Must be in range 0 -> 100, so scale down from 255
    XYZ[2] = Z / 255 * 100
    XYZ[0] = XYZ[0] / 95.047  # ref_X =  95.047   Observer= 2°, Illuminant= D65
    XYZ[1] = XYZ[1] / 100.0  # ref_Y = 100.000
    XYZ[2] = XYZ[2] / 108.883  # ref_Z = 108.883
    num = 0
    for value in XYZ:
        if value > 0.008856:
            value = value ** (0.3333333333333333)
        else:
            value = (7.787 * value) + (16 / 116)
        XYZ[num] = value
        num = num + 1

    # L, A, B, values calculated below
    L = (116 * XYZ[1]) - 16
    a = 500 * (XYZ[0] - XYZ[1])
    b = 200 * (XYZ[1] - XYZ[2])

    return [L, a, b]



# SPDX-License-Identifier: GPL-2.0-or-later
#
# Copyright (C) 2024, Ideas on Board
#
# Lux module for tuning rkisp1

from .lux import Lux


class LuxRkISP1(Lux):
    hr_name = 'Lux (RkISP1)'
    out_name = 'Lux'

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    # We don't need anything from the config file.
    def validate_config(self, config: dict) -> bool:
        return True

    def process(self, config: dict, images: list, outputs: dict) -> dict:
        return self.calculate_lux_reference_values(images)



# SPDX-License-Identifier: GPL-2.0-or-later
#
# Copyright (C) 2019, Raspberry Pi Ltd
# Copyright (C) 2025, Ideas on Board
#
# Base Lux tuning module

from ..module import Module

import logging
import numpy as np

logger = logging.getLogger(__name__)


class Lux(Module):
    type = 'lux'
    hr_name = 'Lux (Base)'
    out_name = 'GenericLux'

    def __init__(self, debug: list):
        super().__init__()

        self.debug = debug

    def calculate_lux_reference_values(self, images):
        # The lux calibration is done on a single image. For best effects, the
        # image with lux level closest to 1000 is chosen.
        imgs = [img for img in images if img.macbeth is not None]
        lux_values = [img.lux for img in imgs]
        index = lux_values.index(min(lux_values, key=lambda l: abs(1000 - l)))
        img = imgs[index]
        logger.info(f'Selected image {img.name} for lux calibration')

        if img.lux < 50:
            logger.warning(f'A Lux level of {img.lux} is very low for proper lux calibration')

        ref_y = self.calculate_y(img)
        exposure_time = img.exposure
        gain = img.againQ8_norm
        aperture = 1
        logger.info(f'RefY:{ref_y} Exposure time:{exposure_time}µs Gain:{gain} Aperture:{aperture}')
        return {'referenceY': ref_y,
                'referenceExposureTime': exposure_time,
                'referenceAnalogueGain': gain,
                'referenceDigitalGain': 1.0,
                'referenceLux': img.lux}

    def calculate_y(self, img):
        max16Bit = 0xffff
        # Average over all grey patches.
        ap_r = np.mean(img.patches[0][3::4]) / max16Bit
        ap_g = (np.mean(img.patches[1][3::4]) + np.mean(img.patches[2][3::4])) / 2 / max16Bit
        ap_b = np.mean(img.patches[3][3::4]) / max16Bit
        logger.debug(f'Averaged grey patches: Red: {ap_r}, Green: {ap_g}, Blue: {ap_b}')

        # Calculate white balance gains.
        gr = ap_g / ap_r
        gb = ap_g / ap_b
        logger.debug(f'WB gains: Red: {gr} Blue: {gb}')

        # Calculate the mean Y value of the whole image
        a_r = np.mean(img.channels[0]) * gr
        a_g = (np.mean(img.channels[1]) + np.mean(img.channels[2])) / 2
        a_b = np.mean(img.channels[3]) * gb
        y = 0.299 * a_r + 0.587 * a_g + 0.114 * a_b
        y /= max16Bit

        return y




# SPDX-License-Identifier: GPL-2.0-or-later
#
# Copyright (C) 2024, Ideas On Board
#
# AWB module for tuning rkisp1

from .awb import AWB

class AWBRkISP1(AWB):
    hr_name = 'AWB (RkISP1)'
    out_name = 'Awb'

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def validate_config(self, config: dict) -> bool:
        return True

    def process(self, config: dict, images: list, outputs: dict) -> dict:
        if not 'awb' in config['general']:
            raise ValueError('AWB configuration missing')
        awb_config = config['general']['awb']
        algorithm = awb_config['algorithm']

        output = {'algorithm': algorithm}
        data = self.do_calculation(images)
        if algorithm == 'grey':
            output['colourGains'] = data['colourGains']
        elif algorithm == 'bayes':
            output['AwbMode'] = awb_config['AwbMode']
            output['priors'] = awb_config['priors']
            output.update(data)
        else:
            raise ValueError(f"Unknown AWB algorithm {output['algorithm']}")

        return output
